//----------------------------------------------------------------
// 日時 <-> 1970/1/1 00:00:00からの秒数(UNIXの日付) 相互変換
//----------------------------------------------------------------
#define SECONDS_IN_DAY   86400   // 24 * 60 * 60
#define SECONDS_IN_HOUR  3600    // 60 * 60

typedef struct {
  int  year;
  byte month;
  byte day;
  byte hour;
  byte min;
  byte sec;
} date_time;

date_time tm;
void unix_time_to_date( unsigned long unix_datetime, date_time *tm );

//----------------------------------------------------------------
// 西暦０年１月１日からの日数を返す
//----------------------------------------------------------------
unsigned long calc_0_days(int year, byte month, byte day) {
  unsigned long days;
  int daysinmonth_ruiseki[12] = {0,31,59,90,120,151,181,212,243,273,304,334};
  
  year--; // 当年は含まず 
  days = (unsigned long)year * 365;
  days += year / 4;  // 閏年の日数を足しこむ
  days -= year/100;  // 閏年で無い日数を差し引く
  days += year/400;  // 差し引きすぎた日数を足しこむ
  days += (unsigned long)daysinmonth_ruiseki[month-1];
  if( is_leapyear( year ) &&  3 <= month ) {
    day++;
  }
  days += (unsigned long)day;
  
  return days;
}

//----------------------------------------------------------------
// 西暦１９７０年１月１日からの日数を返す
//----------------------------------------------------------------
unsigned long calc_unix_days(int year, byte month, byte day) {
  unsigned long days;

  return calc_0_days(year, month, day) - calc_0_days(1970, 1, 1);
}

//----------------------------------------------------------------
// 西暦１９７０年１月１日　００：００：００　からの秒数を返す
//----------------------------------------------------------------
unsigned long calc_unix_seconds(int year, byte month, byte day, byte hour, byte minutes, byte second) {
  unsigned long days;
  unsigned long seconds;

  days = calc_unix_days(year, month, day);
  seconds = days * SECONDS_IN_DAY;
  seconds += (unsigned long)hour * SECONDS_IN_HOUR;
  seconds += (unsigned long)minutes * 60;
  seconds += (unsigned long)second;
  seconds += -9 * SECONDS_IN_HOUR; // JPN(GMT+9) 日本時間

  return seconds;
}

//----------------------------------------------------------------
// 閏年か否かを返す
//----------------------------------------------------------------
boolean is_leapyear( int year )
{
  if( (year % 400) == 0 || ((year % 4) == 0 && (year % 100) != 0)) {
    return true;
  } else {
    return false;
  }
}

//----------------------------------------------------------------
// Unixの日付を日時に変換　ループ処理しているので2msec程度かかる
//----------------------------------------------------------------
void unix_time_to_date( unsigned long unix_datetime, date_time *tm )
{
  tm->year  = 1970;
  tm->month = 1;
  tm->day   = 1;
  tm->hour  = 0;
  tm->min   = 0;
  tm->sec   = 0;
  
  unsigned long yearindate;
  unsigned long seconds_year;
  unsigned long seconds_month;
  byte daysinmonth[12] = {31,28,31,30,31,30,31,31,30,31,30,31};

  unix_datetime += 9 * SECONDS_IN_HOUR;
  while( unix_datetime > 0 ) {
    if( is_leapyear(tm->year) ) {
      yearindate = 366;
      daysinmonth[1] = 29;
    } else {
      yearindate = 365;
      daysinmonth[1] = 28;
    }
    seconds_year  = yearindate * SECONDS_IN_DAY;
    seconds_month = daysinmonth[tm->month - 1] * SECONDS_IN_DAY;
    if( unix_datetime >= seconds_year ) {
      tm->year++;
      unix_datetime -= seconds_year;
    } else if( unix_datetime >= seconds_month ) {
      tm->month++;
      unix_datetime -= seconds_month;
    } else if( unix_datetime >= SECONDS_IN_DAY ) {
      tm->day++;
      unix_datetime -= SECONDS_IN_DAY;  
    } else if( unix_datetime >= SECONDS_IN_HOUR ) {
      tm->hour++;
      unix_datetime -= SECONDS_IN_HOUR; 
    } else if( unix_datetime >= 60 ) {
      tm->min++;
      unix_datetime -= 60;
    } else {
      tm->sec = (byte)unix_datetime;
      unix_datetime= 0;
    }
  }
}

//---------------------------------------------------------------------------
// arduino setup
//---------------------------------------------------------------------------
void setup() {
  unsigned long days;
  unsigned long unix_seconds;
  long a,b;

  Serial.begin(9600);

  unix_seconds = calc_unix_seconds( 2014,7,18,1,2,3 );
  Serial.println(unix_seconds);
  unix_time_to_date( unix_seconds, &tm );
  
  Serial.println(tm.year);
  Serial.println(tm.month);  
  Serial.println(tm.day);  
  Serial.println(tm.hour);  
  Serial.println(tm.min);  
  Serial.println(tm.sec); 
}

//---------------------------------------------------------------------------
// arduino loop
//---------------------------------------------------------------------------
void loop() {
   //  nothing to do       
}

//--------- E N D -----------------------------------------------------------

